@@ -7,9 +7,9 @@ use crate::{
     rfc6979, Error, Signature, SignatureSize,
 };
 use elliptic_curve::{
-    ff::PrimeField, generic_array::ArrayLength, ops::Invert, subtle::ConstantTimeEq,
-    weierstrass::Curve, zeroize::Zeroize, FieldBytes, NonZeroScalar, Order, ProjectiveArithmetic,
-    Scalar, SecretKey,
+    generic_array::ArrayLength, group::ff::PrimeField, ops::Invert, weierstrass::Curve,
+    zeroize::Zeroize, FieldBytes, FieldSize, NonZeroScalar, ProjectiveArithmetic, Scalar,
+    SecretKey,
 };
 use signature::{
     digest::{BlockInput, Digest, FixedOutput, Reset, Update},
@@ -20,8 +20,7 @@ use signature::{
 #[cfg(feature = "verify")]
 use {
     crate::verify::VerifyingKey,
-    core::fmt::Debug,
-    elliptic_curve::{AffinePoint, ProjectivePoint},
+    elliptic_curve::{AffinePoint, PublicKey},
 };
 
 #[cfg(feature = "pkcs8")]
@@ -43,109 +42,96 @@ use core::str::FromStr;
 #[cfg_attr(docsrs, doc(cfg(feature = "sign")))]
 pub struct SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
-    inner: SecretKey<C>,
+    inner: NonZeroScalar<C>,
 }
 
 impl<C> SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     /// Generate a cryptographically random [`SigningKey`].
     pub fn random(rng: impl CryptoRng + RngCore) -> Self {
         Self {
-            inner: SecretKey::random(rng),
+            inner: NonZeroScalar::random(rng),
         }
     }
 
     /// Initialize signing key from a raw scalar serialized as a byte slice.
     pub fn from_bytes(bytes: &[u8]) -> Result<Self, Error> {
-        SecretKey::from_bytes(bytes)
-            .map(|sk| Self { inner: sk })
-            .map_err(|_| Error::new())
+        let inner = SecretKey::from_bytes(bytes)
+            .map(|sk| sk.to_secret_scalar())
+            .map_err(|_| Error::new())?;
+
+        Ok(Self { inner })
     }
 
     /// Get the [`VerifyingKey`] which corresponds to this [`SigningKey`]
     #[cfg(feature = "verify")]
     #[cfg_attr(docsrs, doc(cfg(feature = "verify")))]
-    pub fn verifying_key(&self) -> VerifyingKey<C>
-    where
-        AffinePoint<C>: Copy + Clone + Debug + Default,
-        ProjectivePoint<C>: From<AffinePoint<C>>,
-    {
+    pub fn verifying_key(&self) -> VerifyingKey<C> {
         VerifyingKey {
-            inner: self.inner.public_key(),
+            inner: PublicKey::from_secret_scalar(&self.inner),
         }
     }
 
     /// Serialize this [`SigningKey`] as bytes
     pub fn to_bytes(&self) -> FieldBytes<C> {
-        self.inner.to_bytes()
+        self.inner.to_repr()
     }
 }
 
 impl<C> From<SecretKey<C>> for SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + ConstantTimeEq
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn from(secret_key: SecretKey<C>) -> Self {
-        Self { inner: secret_key }
+        Self::from(&secret_key)
+    }
+}
+
+impl<C> From<&SecretKey<C>> for SigningKey<C>
+where
+    C: Curve + ProjectiveArithmetic,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
+    SignatureSize<C>: ArrayLength<u8>,
+{
+    fn from(secret_key: &SecretKey<C>) -> Self {
+        Self {
+            inner: secret_key.to_secret_scalar(),
+        }
     }
 }
 
 impl<C, D> DigestSigner<D, Signature<C>> for SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    D: FixedOutput<OutputSize = C::FieldSize> + BlockInput + Clone + Default + Reset + Update,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    D: FixedOutput<OutputSize = FieldSize<C>> + BlockInput + Clone + Default + Reset + Update,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     /// Sign message prehash using a deterministic ephemeral scalar (`k`)
     /// computed using the algorithm described in RFC 6979 (Section 3.2):
     /// <https://tools.ietf.org/html/rfc6979#section-3>
     fn try_sign_digest(&self, digest: D) -> Result<Signature<C>, Error> {
-        let k = rfc6979::generate_k(self.inner.secret_scalar(), digest.clone(), &[]);
+        let k = rfc6979::generate_k(&self.inner, digest.clone(), &[]);
         let msg_scalar = Scalar::<C>::from_digest(digest);
-
-        self.inner
-            .secret_scalar()
-            .try_sign_prehashed(&**k, &msg_scalar)
+        self.inner.try_sign_prehashed(&**k, &msg_scalar)
     }
 }
 
 impl<C> signature::Signer<Signature<C>> for SigningKey<C>
 where
     Self: DigestSigner<C::Digest, Signature<C>>,
-    C: Curve + Order + ProjectiveArithmetic + DigestPrimitive,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic + DigestPrimitive,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn try_sign(&self, msg: &[u8]) -> Result<Signature<C>, signature::Error> {
@@ -155,13 +141,9 @@ where
 
 impl<C, D> RandomizedDigestSigner<D, Signature<C>> for SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    D: FixedOutput<OutputSize = C::FieldSize> + BlockInput + Clone + Default + Reset + Update,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    D: FixedOutput<OutputSize = FieldSize<C>> + BlockInput + Clone + Default + Reset + Update,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     /// Sign message prehash using an ephemeral scalar (`k`) derived according
@@ -175,24 +157,17 @@ where
         let mut added_entropy = FieldBytes::<C>::default();
         rng.fill_bytes(&mut added_entropy);
 
-        let k = rfc6979::generate_k(self.inner.secret_scalar(), digest.clone(), &added_entropy);
+        let k = rfc6979::generate_k(&self.inner, digest.clone(), &added_entropy);
         let msg_scalar = Scalar::<C>::from_digest(digest);
-
-        self.inner
-            .secret_scalar()
-            .try_sign_prehashed(&**k, &msg_scalar)
+        self.inner.try_sign_prehashed(&**k, &msg_scalar)
     }
 }
 
 impl<C> RandomizedSigner<Signature<C>> for SigningKey<C>
 where
     Self: RandomizedDigestSigner<C::Digest, Signature<C>>,
-    C: Curve + Order + ProjectiveArithmetic + DigestPrimitive,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic + DigestPrimitive,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn try_sign_with_rng(
@@ -206,17 +181,13 @@ where
 
 impl<C> From<NonZeroScalar<C>> for SigningKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn from(secret_scalar: NonZeroScalar<C>) -> Self {
         Self {
-            inner: SecretKey::new(secret_scalar),
+            inner: secret_scalar,
         }
     }
 }
@@ -224,14 +195,9 @@ where
 #[cfg(feature = "verify")]
 impl<C> From<&SigningKey<C>> for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + ProjectiveArithmetic,
+
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn from(signing_key: &SigningKey<C>) -> VerifyingKey<C> {
@@ -243,14 +209,9 @@ where
 #[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
 impl<C> FromPrivateKey for SigningKey<C>
 where
-    C: Curve + AlgorithmParameters + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + AlgorithmParameters + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
@@ -258,7 +219,7 @@ where
     fn from_pkcs8_private_key_info(
         private_key_info: pkcs8::PrivateKeyInfo<'_>,
     ) -> pkcs8::Result<Self> {
-        SecretKey::from_pkcs8_private_key_info(private_key_info).map(|inner| Self { inner })
+        SecretKey::from_pkcs8_private_key_info(private_key_info).map(Into::into)
     }
 }
 
@@ -266,14 +227,9 @@ where
 #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
 impl<C> FromStr for SigningKey<C>
 where
-    C: Curve + AlgorithmParameters + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>
-        + FromDigest<C>
-        + Invert<Output = Scalar<C>>
-        + SignPrimitive<C>
-        + Zeroize,
+    C: Curve + AlgorithmParameters + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
+    Scalar<C>: FromDigest<C> + Invert<Output = Scalar<C>> + SignPrimitive<C> + Zeroize,
     SignatureSize<C>: ArrayLength<u8>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
