@@ -7,13 +7,12 @@ use crate::{
 use core::{cmp::Ordering, fmt::Debug, ops::Add};
 use elliptic_curve::{
     consts::U1,
-    ff::PrimeField,
     generic_array::ArrayLength,
     sec1::{
         EncodedPoint, FromEncodedPoint, ToEncodedPoint, UncompressedPointSize, UntaggedPointSize,
     },
     weierstrass::{Curve, PointCompression},
-    AffinePoint, FieldBytes, Order, ProjectiveArithmetic, ProjectivePoint, PublicKey, Scalar,
+    AffinePoint, FieldSize, ProjectiveArithmetic, PublicKey, Scalar,
 };
 use signature::{digest::Digest, DigestVerifier};
 
@@ -34,19 +33,15 @@ use core::str::FromStr;
 #[derive(Copy, Clone, Debug)]
 pub struct VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
 {
     pub(crate) inner: PublicKey<C>,
 }
 
 impl<C> VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -73,11 +68,10 @@ where
 
 impl<C, D> DigestVerifier<D, Signature<C>> for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    D: Digest<OutputSize = C::FieldSize>,
-    AffinePoint<C>: Copy + Clone + Debug + VerifyPrimitive<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>> + FromDigest<C>,
+    C: Curve + ProjectiveArithmetic,
+    D: Digest<OutputSize = FieldSize<C>>,
+    AffinePoint<C>: VerifyPrimitive<C>,
+    Scalar<C>: FromDigest<C>,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn verify_digest(&self, digest: D, signature: &Signature<C>) -> Result<(), Error> {
@@ -89,11 +83,10 @@ where
 
 impl<C> signature::Verifier<Signature<C>> for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic + DigestPrimitive,
-    C::Digest: Digest<OutputSize = C::FieldSize>,
-    AffinePoint<C>: Copy + Clone + Debug + VerifyPrimitive<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>> + FromDigest<C>,
+    C: Curve + ProjectiveArithmetic + DigestPrimitive,
+    C::Digest: Digest<OutputSize = FieldSize<C>>,
+    AffinePoint<C>: VerifyPrimitive<C>,
+    Scalar<C>: FromDigest<C>,
     SignatureSize<C>: ArrayLength<u8>,
 {
     fn verify(&self, msg: &[u8], signature: &Signature<C>) -> Result<(), Error> {
@@ -103,10 +96,8 @@ where
 
 impl<C> From<&VerifyingKey<C>> for EncodedPoint<C>
 where
-    C: Curve + Order + ProjectiveArithmetic + PointCompression,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic + PointCompression,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -117,9 +108,7 @@ where
 
 impl<C> From<PublicKey<C>> for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
 {
     fn from(public_key: PublicKey<C>) -> VerifyingKey<C> {
         VerifyingKey { inner: public_key }
@@ -128,9 +117,7 @@ where
 
 impl<C> From<&PublicKey<C>> for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
 {
     fn from(public_key: &PublicKey<C>) -> VerifyingKey<C> {
         public_key.clone().into()
@@ -139,9 +126,7 @@ where
 
 impl<C> From<VerifyingKey<C>> for PublicKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
 {
     fn from(verifying_key: VerifyingKey<C>) -> PublicKey<C> {
         verifying_key.inner
@@ -150,9 +135,7 @@ where
 
 impl<C> From<&VerifyingKey<C>> for PublicKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
 {
     fn from(verifying_key: &VerifyingKey<C>) -> PublicKey<C> {
         verifying_key.clone().into()
@@ -161,10 +144,8 @@ where
 
 impl<C> Eq for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -172,10 +153,8 @@ where
 
 impl<C> PartialEq for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -186,10 +165,8 @@ where
 
 impl<C> PartialOrd for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -200,10 +177,8 @@ where
 
 impl<C> Ord for VerifyingKey<C>
 where
-    C: Curve + Order + ProjectiveArithmetic,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + ProjectiveArithmetic,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -216,10 +191,8 @@ where
 #[cfg_attr(docsrs, doc(cfg(feature = "pkcs8")))]
 impl<C> FromPublicKey for VerifyingKey<C>
 where
-    C: Curve + AlgorithmParameters + Order + ProjectiveArithmetic + PointCompression,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + AlgorithmParameters + ProjectiveArithmetic + PointCompression,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
@@ -232,10 +205,8 @@ where
 #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
 impl<C> FromStr for VerifyingKey<C>
 where
-    C: Curve + AlgorithmParameters + Order + ProjectiveArithmetic + PointCompression,
-    AffinePoint<C>: Copy + Clone + Debug + Default + FromEncodedPoint<C> + ToEncodedPoint<C>,
-    ProjectivePoint<C>: From<AffinePoint<C>>,
-    Scalar<C>: PrimeField<Repr = FieldBytes<C>>,
+    C: Curve + AlgorithmParameters + ProjectiveArithmetic + PointCompression,
+    AffinePoint<C>: FromEncodedPoint<C> + ToEncodedPoint<C>,
     UntaggedPointSize<C>: Add<U1> + ArrayLength<u8>,
     UncompressedPointSize<C>: ArrayLength<u8>,
 {
